import React from 'react';
export declare type VerticalSplitProps = {
    mode?: 'overlay' | 'split';
    minSize?: number | string;
    maxSize?: number | string;
    size?: number | string;
    primaryPane: 'left' | 'right';
    isSecondaryCollapsed?: boolean;
    allowResize?: boolean;
    children: [React.ReactNode, React.ReactNode];
    onDragFinished?: (newSize: number) => void;
    onSecondaryToggle?: (secondaryExpanded: boolean) => void;
};
/**
 * @example Resizable, left primary panel
 *
 * ```js
 * <Canvas ctx={ctx}>
 *   <div style={{ height: 500, position: 'relative' }}>
 *     <VerticalSplit primaryPane="left" size="25%" minSize={220}>
 *       <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
 *         <Toolbar>
 *           <ToolbarStack stackSize="l">
 *             <ToolbarTitle>Primary</ToolbarTitle>
 *           </ToolbarStack>
 *         </Toolbar>
 *         <div
 *           style={{
 *             flex: '1',
 *             display: 'flex',
 *             justifyContent: 'center',
 *             alignItems: 'center',
 *             height: '150px',
 *           }}
 *         >
 *           Main content
 *         </div>
 *       </div>
 *       <div style={{ display: 'flex', flexDirection: 'column', height: '100%', borderLeft: '1px solid var(--border-color)' }}>
 *         <Toolbar>
 *           <ToolbarStack stackSize="l">
 *             <ToolbarTitle>Secondary</ToolbarTitle>
 *           </ToolbarStack>
 *         </Toolbar>
 *         <div
 *           style={{
 *             flex: '1',
 *             display: 'flex',
 *             justifyContent: 'center',
 *             alignItems: 'center',
 *             height: '150px',
 *           }}
 *         >
 *           Sidebar
 *         </div>
 *       </div>
 *     </VerticalSplit>
 *   </div>
 * </Canvas>;
 * ```
 *
 * @example Resizable, right primary panel
 *
 * ```js
 * <Canvas ctx={ctx}>
 *   <div style={{ height: 500, position: 'relative' }}>
 *     <VerticalSplit primaryPane="right" size="25%" minSize={220}>
 *       <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
 *         <Toolbar>
 *           <ToolbarStack stackSize="l">
 *             <ToolbarTitle>Secondary</ToolbarTitle>
 *           </ToolbarStack>
 *         </Toolbar>
 *         <div
 *           style={{
 *             flex: '1',
 *             display: 'flex',
 *             justifyContent: 'center',
 *             alignItems: 'center',
 *             height: '150px',
 *           }}
 *         >
 *           Sidebar
 *         </div>
 *       </div>
 *       <div style={{ display: 'flex', flexDirection: 'column', height: '100%', borderLeft: '1px solid var(--border-color)' }}>
 *         <Toolbar>
 *           <ToolbarStack stackSize="l">
 *             <ToolbarTitle>Primary</ToolbarTitle>
 *           </ToolbarStack>
 *         </Toolbar>
 *         <div
 *           style={{
 *             flex: '1',
 *             display: 'flex',
 *             justifyContent: 'center',
 *             alignItems: 'center',
 *             height: '150px',
 *           }}
 *         >
 *           Main content
 *         </div>
 *       </div>
 *     </VerticalSplit>
 *   </div>
 * </Canvas>;
 * ```
 *
 * @example Collapsible
 *
 * ```js
 *   <Canvas ctx={ctx}>
 *    <div style={{ height: 500, position: 'relative' }}>
 *      <StateManager initial={true}>
 *        {(isCollapsed, setCollapsed) => (
 *          <VerticalSplit
 *            primaryPane="left"
 *            size="25%"
 *            minSize={220}
 *            isSecondaryCollapsed={isCollapsed}
 *            onSecondaryToggle={setCollapsed}
 *          >
 *            <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
 *              <Toolbar>
 *                <ToolbarStack stackSize="l">
 *                  <ToolbarTitle>Primary</ToolbarTitle>
 *                </ToolbarStack>
 *              </Toolbar>
 *              <div
 *                style={{
 *                  flex: '1',
 *                  display: 'flex',
 *                  justifyContent: 'center',
 *                  alignItems: 'center',
 *                  height: '150px',
 *                }}
 *              >
 *                Main content
 *              </div>
 *            </div>
 *            <div
 *              style={{
 *                display: 'flex',
 *                flexDirection: 'column',
 *                height: '100%',
 *                borderLeft: '1px solid var(--border-color)',
 *              }}
 *            >
 *              <Toolbar>
 *                <ToolbarStack stackSize="l">
 *                  <ToolbarTitle>Secondary</ToolbarTitle>
 *                </ToolbarStack>
 *              </Toolbar>
 *              <div
 *                style={{
 *                  flex: '1',
 *                  display: 'flex',
 *                  justifyContent: 'center',
 *                  alignItems: 'center',
 *                  height: '150px',
 *                }}
 *              >
 *                Sidebar
 *              </div>
 *            </div>
 *          </VerticalSplit>
 *        )}
 *      </StateManager>
 *    </div>
 *  </Canvas>;
 * ```
 *
 * @example Overlay mode
 *
 * ```js
 *   <Canvas ctx={ctx}>
 *    <div style={{ height: 500, position: 'relative' }}>
 *      <StateManager initial={true}>
 *        {(isCollapsed, setCollapsed) => (
 *          <VerticalSplit
 *            mode="overlay"
 *            primaryPane="left"
 *            size="25%"
 *            minSize={220}
 *            isSecondaryCollapsed={isCollapsed}
 *            onSecondaryToggle={setCollapsed}
 *          >
 *            <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
 *              <Toolbar>
 *                <ToolbarStack stackSize="l">
 *                  <ToolbarTitle>Primary</ToolbarTitle>
 *                </ToolbarStack>
 *              </Toolbar>
 *              <div
 *                style={{
 *                  flex: '1',
 *                  display: 'flex',
 *                  justifyContent: 'center',
 *                  alignItems: 'center',
 *                  height: '150px',
 *                }}
 *              >
 *                Main content
 *              </div>
 *            </div>
 *            <div
 *              style={{
 *                display: 'flex',
 *                flexDirection: 'column',
 *                height: '100%',
 *                borderLeft: '1px solid var(--border-color)',
 *              }}
 *            >
 *              <Toolbar>
 *                <ToolbarStack stackSize="l">
 *                  <ToolbarTitle>Secondary</ToolbarTitle>
 *                </ToolbarStack>
 *              </Toolbar>
 *              <div
 *                style={{
 *                  flex: '1',
 *                  display: 'flex',
 *                  justifyContent: 'center',
 *                  alignItems: 'center',
 *                  height: '150px',
 *                }}
 *              >
 *                Sidebar
 *              </div>
 *            </div>
 *          </VerticalSplit>
 *        )}
 *      </StateManager>
 *    </div>
 *  </Canvas>;
 * ```
 */
export declare function VerticalSplit({ mode, minSize, maxSize, size, primaryPane, children, allowResize, onDragFinished, onSecondaryToggle, isSecondaryCollapsed, }: VerticalSplitProps): JSX.Element;
